/***************************************************************************
 *                                                                         *
 *   kemboss - Graphical user interface for EMBOSS                         *
 *                                                                         *
 *   Copyright (C) 2007-2013 Tobias Schwarz (mail@tobias-schwarz.net)      *
 *                                                                         *
 *   Based on the EMBOSS.kaptn scripts by Thomas Siegmund                  *
 *                                                                         *
 *   This program is free software; you can redistribute and/or modify     *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version, and provided that the above       *
 *   copyright and permission notice is included with all distributed      *
 *   copies of this or derived software.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 **************************************************************************/

#ifndef HIGHLIGHTER_H
#define HIGHLIGHTER_H

#include <QSyntaxHighlighter>
#include <QHash>
#include "settings.h"

class QTextDocument;

class Highlighter : public QSyntaxHighlighter
{

Q_OBJECT

public:
   Highlighter(QTextDocument *parent = 0, kembossSettings *fksettings = 0);
   virtual ~Highlighter() {}
   enum sequenceType { noSeq = 0,
                       anySeq = 1,
                       aaSeq = 2,
                       nucSeq = 3
   };

   enum sequenceSyntax { none = 0,
                         FastaAA = 1,
                         FastaNuc = 2,
                         AlignmentAA = 3,
                         AlignmentNuc = 4,
                         EMBL = 5,
                         GenbankAA = 6,
                         GenbankNuc = 7,
                         Swiss = 8
   };

   sequenceSyntax syntax;
   void setSyntax(const sequenceSyntax newSyntax);
   void setSyntaxInt(const int newSyntax);

protected:
   void highlightBlock(const QString &text);

private:
   void highlightAASequence(const QString &text, const int start, const int end);
   void highlightNucSequence(const QString &text, const int start, const int end);
   kembossSettings *ksettings;
   struct HighlightingRule
   {
      sequenceType type;
      QRegExp pattern;
      QTextCharFormat format;
   };

   QRegExp AAStart;
   QRegExp AAEnd;

   QVector<HighlightingRule> FASTARules;
   QVector<HighlightingRule> EMBLRules;
   QVector<HighlightingRule> AlignmentRules;
   QVector<HighlightingRule> GbRules;
   QVector<HighlightingRule> SwissRules;

   QVector<HighlightingRule> aaSequenceRules;
   QVector<HighlightingRule> nucSequenceRules;
};

#endif
